#ifndef __DGLBASICSHAPE_HPP__
#define __DGLBASICSHAPE_HPP__

#include <GL/dglcommon.hpp>

namespace DECS{
  namespace GL{

    class DGLLine : DGLObject{
    private:
      DGLPosition beginPoint_;
      DGLPosition endPoint_;

      DGLLine(){};
    public:
      DGLLine(DGLPosition beginPoint, DGLPosition endPoint):
	DGLObject(GL_LINES),
	beginPoint_(beginPoint), endPoint_(endPoint){};
      DGLLine(DGLPosition beginPoint, DGLPosition endPoint, DGLColor color):
	DGLObject(GL_LINES, color),
	beginPoint_(beginPoint), endPoint_(endPoint){};
      DGLLine(int mode,
	      DGLPosition beginPoint, DGLPosition endPoint,
	      DGLColor color):
	DGLObject(mode, color),
	beginPoint_(beginPoint), endPoint_(endPoint){};
      ~DGLLine(){};

      void setPoint(DGLPosition beginPoint, DGLPosition endPosition);
      virtual void setColor(DGLColor color);
      virtual void draw();
    };
    
    class DGLTriangle : DGLObject{
    private:
      DGLPosition point1_;
      DGLPosition point2_;
      DGLPosition point3_;
      
      DGLTriangle(){};
    public:
      DGLTriangle(DGLPosition point1, DGLPosition point2, DGLPosition point3):
	DGLObject(GL_LINE_LOOP),
	point1_(point1), point2_(point2), point3_(point3){};
      DGLTriangle(DGLPosition point1, DGLPosition point2, DGLPosition point3,
		  DGLColor color):
	DGLObject(GL_LINE_LOOP, color),
	point1_(point1), point2_(point2), point3_(point3){}; 
      DGLTriangle(int mode,
		  DGLPosition point1, DGLPosition point2, DGLPosition point3):
	DGLObject(mode),
	point1_(point1), point2_(point2), point3_(point3){}; 
      DGLTriangle(int mode,
		  DGLPosition point1, DGLPosition point2, DGLPosition point3,
		  DGLColor color):
	DGLObject(mode, color),
	point1_(point1), point2_(point2), point3_(point3){};  
      ~DGLTriangle(){};
 
      void setPoint(DGLPosition point1, DGLPosition point2, DGLPosition point3);
      virtual void setColor(DGLColor color);
      virtual void draw();
    };

    class DGLSquare : DGLObject{
    private:
      DGLPosition point1_;
      DGLPosition point2_;
      DGLPosition point3_;
      DGLPosition point4_;

      DGLSquare(){};
    public:
      DGLSquare(DGLPosition point1, DGLPosition point2,
		DGLPosition point3, DGLPosition point4):
	DGLObject(GL_LINE_LOOP),
	point1_(point1), point2_(point2),
	point3_(point3), point4_(point4){};
      DGLSquare(DGLPosition point1, DGLPosition point2,
		DGLPosition point3, DGLPosition point4,
		DGLColor color):
	DGLObject(GL_LINE_LOOP, color),
	point1_(point1), point2_(point2), 
	point3_(point3), point4_(point4){}; 
      DGLSquare(int mode,
		DGLPosition point1, DGLPosition point2,
		DGLPosition point3, DGLPosition point4):
	DGLObject(mode),
	point1_(point1), point2_(point2),
	point3_(point3), point4_(point4){};  
      DGLSquare(int mode,
		DGLPosition point1, DGLPosition point2,
		DGLPosition point3, DGLPosition point4,
		DGLColor color):
	DGLObject(mode, color),
	point1_(point1), point2_(point2),
	point3_(point3), point4_(point4){};  
      ~DGLSquare(){};
 
      void setPoint(DGLPosition point1, DGLPosition point2,
		    DGLPosition point3, DGLPosition point4);
      virtual void setColor(DGLColor color);
      virtual void draw();
    };

    class DGLCircle : DGLObject{
    private:
      DGLPosition center_;
      GLdouble radius_;
      int nPoint_;

      DGLCircle(){};
    public:
      DGLCircle(GLdouble radius):
	DGLObject(GL_LINE_LOOP),
	center_(DGL_ORIGIN), radius_(radius), nPoint_(100){};
      DGLCircle(DGLPosition center, GLdouble radius):
	DGLObject(GL_LINE_LOOP),
	center_(center), radius_(radius), nPoint_(100){};
      DGLCircle(int mode, DGLPosition center, GLdouble radius):
	DGLObject(mode),
	center_(center), radius_(radius), nPoint_(100){};
      DGLCircle(DGLPosition center, GLdouble radius, DGLColor color):
	DGLObject(GL_LINE_LOOP, color),
	center_(center), radius_(radius), nPoint_(100){};
      DGLCircle(int mode,
		DGLPosition center, GLdouble radius, DGLColor color):
	DGLObject(mode, color),
	center_(center), radius_(radius), nPoint_(100){};
      DGLCircle(int mode,
		DGLPosition center, GLdouble radius, int nPoint,
		DGLColor color):
	DGLObject(mode, color),
	center_(center), radius_(radius), nPoint_(nPoint){};
      ~DGLCircle(){};

      void setCenter(DGLPosition center);
      void setRadius(GLdouble radius);
      void setNumberOfPoints(int nPoint);

      virtual void setColor(DGLColor color);
      virtual void draw();
    };
    /* Following 3D Shape Classes are not supported yet.
    class DGLCube : DGLObject{
    };

    class DGLPyramid : DGLObject{
    };

    class DGLCone : DGLObject{
    };

    class DGLSphere : DGLObject{
    };
    */
  }
}

#endif // __GLBASICSHAPE_HPP__
