#ifndef __DGLWINDOW_HPP__
#define __DGLWINDOW_HPP__

#include <string>

#include <GL/dgl.hpp>

using namespace std;

namespace DECS{
  namespace GL{

    typedef void (*RESIZE_FUNC)(int, int);
    RESIZE_FUNC getResizeFunc(GLdouble canvasWidth, GLdouble canvasHeight);

    struct DGLWindowProperty{
      string title_;
      unsigned int mode_;
      DGLColor bgColor_;

      int wWidth_;
      int wHeight_;
      int wPositionX_;
      int wPositionY_;
    };

    static DGLWindowProperty DGL_DEFAULT_WPROP = {
      "DGL** Window", GL_RGBA, DGL_WHITE,
      300, 300, 100, 100};

    class DGLWindow{
    private:
      DGLWindowProperty* wProp_;
      
      void (*displayFunc_)();
      void (*resizeFunc_)(int width, int height);
      void (*mouseFunc_)(int button, int state, int x, int y);

      void init();
    public:
      DGLWindow(int argc, char* argv[]):
	wProp_(&DGL_DEFAULT_WPROP),
	displayFunc_(0), resizeFunc_(0), mouseFunc_(0)
      {
	this->wProp_->wPositionX_ =
	  (glutGet(GLUT_SCREEN_WIDTH) - this->wProp_->wWidth_) / 2;
	this->wProp_->wPositionY_ =
	  (glutGet(GLUT_SCREEN_HEIGHT) - this->wProp_->wHeight_) / 2;
	glutInitWindowPosition(this->wProp_->wPositionX_,
			       this->wProp_->wPositionY_);
	glutInitWindowSize(this->wProp_->wWidth_, this->wProp_->wHeight_);
	glutInit(&argc, argv);
	glutInitDisplayMode(this->wProp_->mode_);
      }

      DGLWindow(int argc, char* argv[],
	       void (*displayFunc)(),
	       void (*resizeFunc)(int width, int height)):
	wProp_(&DGL_DEFAULT_WPROP),
	displayFunc_(displayFunc), resizeFunc_(resizeFunc),
	mouseFunc_(0)
      {
	this->wProp_->wPositionX_ =
	  (glutGet(GLUT_SCREEN_WIDTH) - this->wProp_->wWidth_) / 2;
	this->wProp_->wPositionY_ =
	  (glutGet(GLUT_SCREEN_HEIGHT) - this->wProp_->wHeight_) / 2;
	glutInitWindowPosition(this->wProp_->wPositionX_,
			       this->wProp_->wPositionY_);
	glutInitWindowSize(this->wProp_->wWidth_, this->wProp_->wHeight_);
	glutInit(&argc, argv);
	glutInitDisplayMode(this->wProp_->mode_);
      }

      DGLWindow(int argc, char* argv[], DGLWindowProperty* wProp):
	wProp_(wProp),
	displayFunc_(0), resizeFunc_(0), mouseFunc_(0)
      {
	glutInitWindowPosition(this->wProp_->wPositionX_,
			       this->wProp_->wPositionY_);
	glutInitWindowSize(this->wProp_->wWidth_, this->wProp_->wHeight_);
	glutInit(&argc, argv);
	glutInitDisplayMode(this->wProp_->mode_);
      }

      DGLWindow(int argc, char* argv[], DGLWindowProperty* wProp,
	       void (*displayFunc)(),
	       void (*resizeFunc)(int width, int height)):
	wProp_(wProp),
	displayFunc_(displayFunc), resizeFunc_(resizeFunc),
	mouseFunc_(0)
      {
	glutInitWindowPosition(this->wProp_->wPositionX_,
			       this->wProp_->wPositionY_);
	glutInitWindowSize(this->wProp_->wWidth_, this->wProp_->wHeight_);
	glutInit(&argc, argv);
	glutInitDisplayMode(this->wProp_->mode_);
      }
      
      ~DGLWindow(){};
      
      void setBGColor(DGLColor bgColor);
      void setDisplayFunc(void (*displayFunc)());
      void setResizeFunc(void (*resizeFunc)(int width, int height));
      void setMouseFunc(void (*mouseFunc)(int button, int state, int x, int y));

      void create();
    };
  }
}

#endif // __DGLWINDOW_HPP__
