#include <cmath>

#include <GL/dglbasicshape.hpp>

using namespace std;

namespace DECS{
  namespace GL{
    void DGLLine::setPoint(DGLPosition beginPoint, DGLPosition endPoint){
      this->beginPoint_ = beginPoint;
      this->endPoint_ = endPoint;
    }

    void DGLLine::setColor(DGLColor color){
      this->color_ = color;
    }

    void DGLLine::draw(){
      glColor3d(this->color_.r(), this->color_.g(), this->color_.b());
      glBegin(this->mode_);
      glVertex3d(this->beginPoint_.x(), this->beginPoint_.y(), 0.0);
      glVertex3d(this->endPoint_.x(), this->endPoint_.y(), 0.0);
      glEnd();
    }


    void DGLTriangle::setPoint(DGLPosition point1,
			       DGLPosition point2,
			       DGLPosition point3)
    {
      this->point1_ = point1;
      this->point2_ = point2;
      this->point3_ = point3;
    }

    void DGLTriangle::setColor(DGLColor color)
    {
      this->color_ = color;
    }

    void DGLTriangle::draw()
    {
      glColor3d(this->color_.r(), this->color_.g(), this->color_.b());
      glBegin(this->mode_);
      glVertex3d(this->point1_.x(), this->point1_.y(), 0.0);
      glVertex3d(this->point2_.x(), this->point2_.y(), 0.0);
      glVertex3d(this->point3_.x(), this->point3_.y(), 0.0);
      glEnd();
    }

    void DGLSquare::setPoint(DGLPosition point1, DGLPosition point2,
			     DGLPosition point3, DGLPosition point4)
    {
      this->point1_ = point1;
      this->point2_ = point2;
      this->point3_ = point3;
      this->point4_ = point4;
    }

    void DGLSquare::setColor(DGLColor color)
    {
      this->color_ = color;
    }

    void DGLSquare::draw(){
      glColor3d(this->color_.r(), this->color_.g(), this->color_.b());
      glBegin(this->mode_);
      glVertex3d(this->point1_.x(), this->point1_.y(), 0.0);
      glVertex3d(this->point2_.x(), this->point2_.y(), 0.0);
      glVertex3d(this->point3_.x(), this->point3_.y(), 0.0);
      glVertex3d(this->point4_.x(), this->point4_.y(), 0.0);
      glEnd();
    }


    void DGLCircle::setCenter(DGLPosition center)
    {
      this->center_ = center;
    }

    void DGLCircle::setRadius(GLdouble radius)
    {
      this->radius_ = radius;
    }

    void DGLCircle::setNumberOfPoints(int nPoint)
    {
      this->nPoint_ = nPoint;
    }

    void DGLCircle::setColor(DGLColor color){
      this->color_ = color;
    }

    void DGLCircle::draw(){
      glColor3d(this->color_.r(), this->color_.g(), this->color_.b());
      glBegin(this->mode_);
      for(int i = 0; i < this->nPoint_; i++){
	double rate = (double)i / nPoint_;
	double x = this->radius_ * 2 * cos(2.0 * M_PI * rate)
	  + this->center_.x();
	double y = this->radius_ * 2 * sin(2.0 * M_PI * rate)
	  + this->center_.y();
	glVertex3d(x, y, 0.0);
      }
      glEnd();
    }
  }
}
