#include <GL/dglwindow.hpp>

namespace DECS{
  namespace GL{

    struct CANVAS_SIZE{
      GLdouble cWidth;
      GLdouble cHeight;
    } cSize;

    void resize(int width, int height){
      glViewport(0, 0, width, height);
      glLoadIdentity();
      glOrtho(-width / cSize.cWidth, width / cSize.cWidth,
	      -height / cSize.cHeight, height / cSize.cHeight,
	      -1.0, 1.0);
    }

    RESIZE_FUNC getResizeFunc(GLdouble canvasWidth,
			      GLdouble canvasHeight)
    {
      cSize.cWidth = canvasWidth;
      cSize.cHeight = canvasHeight;
      return resize;
    }

    void DGLWindow::init()
    {
      glClearColor(this->wProp_->bgColor_.r(),
		   this->wProp_->bgColor_.g(),
		   this->wProp_->bgColor_.b(),
		   this->wProp_->bgColor_.a());
    }

    void  DGLWindow::setBGColor(DGLColor bgColor)
    {
      this->wProp_->bgColor_ = bgColor;
    }

    void  DGLWindow::setDisplayFunc(void(*displayFunc)())
    {
      displayFunc_ = displayFunc;
    }

    void  DGLWindow::setResizeFunc(void(*resizeFunc)(int width, int height))
    {
      resizeFunc_ = resizeFunc;
    }

    void  DGLWindow::setMouseFunc(void(*mouseFunc)(int button, 
						  int state, int x, int y))
    {
      mouseFunc_ = mouseFunc;
    }

    void  DGLWindow::create(){
      glutCreateWindow(this->wProp_->title_.c_str());
      if(displayFunc_){
	glutDisplayFunc(this->displayFunc_);
      }
      else{
       	//throw DGLWindowException;
      }
      if(resizeFunc_){
	glutReshapeFunc(this->resizeFunc_);
      }
      else{
	//throw DGLWindowException;
      }
      if(mouseFunc_){
	glutMouseFunc(this->mouseFunc_);
      }
      this->init();
      glutMainLoop();
    }
  }
}
